@@ -28,6 +28,9 @@ module Database.Persist.Class
     -- * PersistConfig
     , PersistConfig (..)
 
+    -- * JSON utilities
+    , keyValueEntityToJSON, keyValueEntityFromJSON
+    , entityIdToJSON, entityIdFromJSON
     ) where
 
 import Database.Persist.Class.DeleteCascade
