@@ -15,17 +15,18 @@ module Database.Persist.Sql
     , getStmtConn
       -- * Internal
     , module Database.Persist.Sql.Internal
+    , decorateSQLWithLimitOffset
     ) where
 
 import Database.Persist
 import Database.Persist.Sql.Types
 import Database.Persist.Sql.Class
-import Database.Persist.Sql.Run
+import Database.Persist.Sql.Run hiding (withResourceTimeout)
 import Database.Persist.Sql.Raw
 import Database.Persist.Sql.Migration
 import Database.Persist.Sql.Internal
 
-import Database.Persist.Sql.Orphan.PersistQuery
+import Database.Persist.Sql.Orphan.PersistQuery 
 import Database.Persist.Sql.Orphan.PersistStore ()
 import Database.Persist.Sql.Orphan.PersistUnique ()
 import Control.Monad.IO.Class
